@@ -434,10 +434,6 @@ class StandardFileLocations(Enum):
     downloads = 8
 
 
-class FileManagerType(Enum):
-    regular = 1
-    select = 2
-
 
 max_remembered_destinations = 10
 
@@ -499,28 +495,68 @@ class Desktop(Enum):
     lxqt = 8
     ubuntugnome = 9
     popgnome = 10
-    unknown = 11
+    deepin = 11
+    unknown = 12
+
+
+class FileManagerType(Enum):
+    regular = 1
+    select = 2
+    dir_only_uri = 3
+    show_item = 4
+
+
+FileManagerBehavior = dict(
+    nautilus=FileManagerType.regular,
+    dolphin=FileManagerType.select,
+    caja=FileManagerType.dir_only_uri,
+    thunar=FileManagerType.dir_only_uri,
+    nemo=FileManagerType.regular,
+    pcmanfm=FileManagerType.dir_only_uri,
+)
+FileManagerBehavior['pcmanfm-qt'] = FileManagerType.dir_only_uri
+FileManagerBehavior['dde-file-manager'] = FileManagerType.show_item
+
+
+DefaultFileBrowserFallback = dict(
+    gnome='nautilus',
+    ubuntugnome='nautilus',
+    popgnome='nautilus',
+    unity='nautilus',
+    kde='dolphin',
+    cinnamon='nemo',
+    mate='caja',
+    xfce='thunar',
+    lxde='pcmanfm',
+    lxqt='pcmanfm-qt',
+    deepin='dde-file-manager',
+)
 
 
 class Distro(Enum):
     debian = 1
     ubuntu = 2
     fedora = 3
-    unknown = 4
-    manjaro = 5
-    arch = 6
-    neon = 7
+    neon = 4
+    linuxmint = 5
+    korora = 6
+    arch = 7
     opensuse = 8
-    gentoo = 9
+    manjaro = 9
     galliumos = 10
     peppermint = 11
     antergos = 12
     elementary = 13
+    centos = 14
+    gentoo = 15
+    deepin = 16
+    unknown = 20
 
 
 orientation_offset = dict(
     arw=106,
     cr2=126,
+    cr3=60000,  # assuming ExifTool (exiv2 >= 0.28 required for CR3)
     dcr=7684,
     dng=144,
     mef=144,
@@ -535,10 +571,34 @@ orientation_offset = dict(
     sr2=82,
     srw=46
 )
+orientation_offset['3fr'] = 132
+
+orientation_offset_exiftool = dict(
+    arw=350,
+    cr2=320,
+    cr3=60000,
+    crw=20,
+    dcr=8196,
+    dng=644,
+    iiq=20,
+    mef=376,
+    mrw=152580,
+    nef=392,
+    nrw=94,
+    orf=6148,
+    pef=332,
+    raf=70660,
+    raw=548,
+    rw2=709636,
+    sr2=276,
+    srw=126
+)
+orientation_offset_exiftool['3fr'] = 376
 
 datetime_offset = dict(
     arw=1540,
     cr2=1028,
+    cr3=60000,  # assuming ExifTool
     dng=119812,
     mef=772,
     mrw=152580,
@@ -558,11 +618,41 @@ datetime_offset = dict(
     avi=50000,
     mov=250000,
 )
+datetime_offset['3fr'] = 1540
 datetime_offset['3gp'] = 5000
 
+datetime_offset_exiftool = dict(
+    arw=1540,
+    cr2=1000,  # varies widely :-/
+    cr3=60000,
+    crw=20,
+    dng=3000,  # varies widely :-/
+    mef=772,
+    mrw=152580,
+    nef=13316,
+    nrw=488,
+    orf=7172,
+    pef=836,
+    raf=70660,
+    raw=932,
+    rw2=709636,
+    sr2=836,
+    srw=496,
+    x3f=69220070,
+    mts=5000,
+    m2t=5000,
+    m2ts=5000,
+    mp4=50000,
+    avi=50000,
+    mov=250000,
+)
+datetime_offset_exiftool['3fr'] = 1042
+datetime_offset_exiftool['3gp'] = 5000
+
 all_tags_offset = dict(
     arw=1848,
     cr2=94622,
+    cr3=60000,  # assuming ExifTool
     dng=143774,
     mef=965,
     mrw=183096,
@@ -570,11 +660,32 @@ all_tags_offset = dict(
     nrw=1848,
     orf=812242,
     pef=1042,
-    raf=12907,
+    raf=13522,
     raw=890885,
     rw2=1205458,
     sr2=1080,
     srw=614,
+)
+all_tags_offset['3fr'] = 1848
+
+all_tags_offset_exiftool = dict(
+    arw=1540,
+    cr2=104453,
+    cr3=60000,
+    dng=143774,
+    dcr=10450,
+    mef=965,
+    mrw=183096,
+    nef=77213623,
+    nrw=1848,
+    orf=29113613,
+    pef=183096,
+    raf=84792,
+    raw=890885,
+    rw2=1205458,
+    sr2=1080,
+    srw=222418,
+    x3f=7380128,
     mp4=130000,
     mts=1300000,
     mt2=1300000,
@@ -582,6 +693,7 @@ all_tags_offset = dict(
     avi=50000,
     mov=250000
 )
+all_tags_offset_exiftool['3fr'] = 1042
 
 thumbnail_offset = dict(
     jpg=100000,
@@ -598,6 +710,30 @@ thumbnail_offset = dict(
     tod=500000,
 )
 
+# Repeat video information here
+thumbnail_offset_exiftool = dict(
+    cr2=694277,
+    cr3=45470,
+    mrw=84792,
+    nef=77213623,
+    nrw=45470,
+    raf=84792,
+    raw=890885,
+    rw2=1205458,
+    sr2=222418,
+    srw=812242,
+    avi=500000,
+    mod=500000,
+    mov=2000000,
+    mp4=2000000,
+    mts=600000,
+    m2t=600000,
+    mpg=500000,
+    mpeg=500000,
+    tod=500000,
+)
+
+
 
 class RememberThisMessage(Enum):
     remember_choice = 1
